@@ -182,12 +182,24 @@ public abstract class JsonParser
          @Deprecated
         ALLOW_NUMERIC_LEADING_ZEROS(false),
 
+        /**
+         * @deprecated Use {@link com.fasterxml.jackson.core.json.JsonReadFeature#ALLOW_LEADING_PLUS_SIGN_FOR_NUMBERS} instead
+         */
+        @Deprecated
+        ALLOW_LEADING_PLUS_SIGN_FOR_NUMBERS(false),
+
         /**
          * @deprecated Use {@link com.fasterxml.jackson.core.json.JsonReadFeature#ALLOW_LEADING_DECIMAL_POINT_FOR_NUMBERS} instead
          */
         @Deprecated
         ALLOW_LEADING_DECIMAL_POINT_FOR_NUMBERS(false),
 
+        /**
+         * @deprecated Use {@link com.fasterxml.jackson.core.json.JsonReadFeature#ALLOW_TRAILING_DECIMAL_POINT_FOR_NUMBERS} instead
+         */
+        @Deprecated
+        ALLOW_TRAILING_DECIMAL_POINT_FOR_NUMBERS(false),
+
         /**
          * Feature that allows parser to recognize set of
          * "Not-a-Number" (NaN) tokens as legal floating number
@@ -325,6 +337,17 @@ public abstract class JsonParser
           */
          INCLUDE_SOURCE_IN_LOCATION(true),
 
+         /**
+          * Feature that determines whether we use the built-in {@link Double#parseDouble(String)} code to parse
+          * doubles or if we use {@link com.fasterxml.jackson.core.io.doubleparser}
+          * instead.
+          *<p>
+          * This setting is disabled by default for backwards compatibility.
+          *
+          * @since 2.14
+          */
+         USE_FAST_DOUBLE_PARSER(false),
+
          ;
 
         /**
@@ -389,7 +412,11 @@ public abstract class JsonParser
     /**********************************************************
      */
 
-    protected JsonParser() { }
+    protected JsonParser() {
+        // @since 2.14 do use sane defaults
+        _features = JsonFactory.DEFAULT_PARSER_FEATURE_FLAGS;
+    }
+
     protected JsonParser(int features) { _features = features; }
 
     /**
@@ -429,43 +456,6 @@ public abstract class JsonParser
      */
     public Object getInputSource() { return null; }
 
-    /**
-     * Helper method, usually equivalent to:
-     *<code>
-     *   getParsingContext().getCurrentValue();
-     *</code>
-     *<p>
-     * Note that "current value" is NOT populated (or used) by Streaming parser;
-     * it is only used by higher-level data-binding functionality.
-     * The reason it is included here is that it can be stored and accessed hierarchically,
-     * and gets passed through data-binding.
-     *
-     * @return "Current value" associated with the current input context (state) of this parser
-     *
-     * @since 2.5
-     */
-    public Object getCurrentValue() {
-        JsonStreamContext ctxt = getParsingContext();
-        return (ctxt == null) ? null : ctxt.getCurrentValue();
-    }
-
-    /**
-     * Helper method, usually equivalent to:
-     *<code>
-     *   getParsingContext().setCurrentValue(v);
-     *</code>
-     *
-     * @param v Current value to assign for the current input context of this parser
-     * 
-     * @since 2.5
-     */
-    public void setCurrentValue(Object v) {
-        JsonStreamContext ctxt = getParsingContext();
-        if (ctxt != null) {
-            ctxt.setCurrentValue(v);
-        }
-    }
-
     /**
      * Sets the payload to be passed if {@link JsonParseException} is thrown.
      *
@@ -682,22 +672,8 @@ public abstract class JsonParser
     public abstract JsonStreamContext getParsingContext();
 
     /**
-     * Method that return the <b>starting</b> location of the current
-     * token; that is, position of the first character from input
-     * that starts the current token.
-     *<p>
-     * Note that the location is not guaranteed to be accurate (although most
-     * implementation will try their best): some implementations may only
-     * return {@link JsonLocation#NA} due to not having access
-     * to input location information (when delegating actual decoding work
-     * to other library)
-     *
-     * @return Starting location of the token parser currently points to
-     */
-    public abstract JsonLocation getTokenLocation();
-
-    /**
-     * Method that returns location of the last processed character;
+     * Method that returns location of the last processed input unit (character
+     * or byte) from the input;
      * usually for error reporting purposes.
      *<p>
      * Note that the location is not guaranteed to be accurate (although most
@@ -708,9 +684,112 @@ public abstract class JsonParser
      * to other library)
      *
      * @return Location of the last processed input unit (byte or character)
+     *
+     * @since 2.13
+     */
+    public JsonLocation currentLocation() {
+        return getCurrentLocation();
+    }
+
+    /**
+     * Method that return the <b>starting</b> location of the current
+     * (most recently returned)
+     * token; that is, the position of the first input unit (character or byte) from input
+     * that starts the current token.
+     *<p>
+     * Note that the location is not guaranteed to be accurate (although most
+     * implementation will try their best): some implementations may only
+     * return {@link JsonLocation#NA} due to not having access
+     * to input location information (when delegating actual decoding work
+     * to other library)
+     *
+     * @return Starting location of the token parser currently points to
+     *
+     * @since 2.13 (will eventually replace {@link #getTokenLocation})
+     */
+    public JsonLocation currentTokenLocation() {
+        return getTokenLocation();
+    }
+
+    // TODO: deprecate in 2.14 or later
+    /**
+     * Alias for {@link #currentLocation()}, to be deprecated in later
+     * Jackson 2.x versions (and removed from Jackson 3.0).
+     *
+     * @return Location of the last processed input unit (byte or character)
      */
     public abstract JsonLocation getCurrentLocation();
 
+    // TODO: deprecate in 2.14 or later
+    /**
+     * Alias for {@link #currentTokenLocation()}, to be deprecated in later
+     * Jackson 2.x versions (and removed from Jackson 3.0).
+     *
+     * @return Starting location of the token parser currently points to
+     */
+    public abstract JsonLocation getTokenLocation();
+
+    /**
+     * Helper method, usually equivalent to:
+     *<code>
+     *   getParsingContext().getCurrentValue();
+     *</code>
+     *<p>
+     * Note that "current value" is NOT populated (or used) by Streaming parser;
+     * it is only used by higher-level data-binding functionality.
+     * The reason it is included here is that it can be stored and accessed hierarchically,
+     * and gets passed through data-binding.
+     *
+     * @return "Current value" associated with the current input context (state) of this parser
+     *
+     * @since 2.13 (added as replacement for older {@link #getCurrentValue()}
+     */
+    public Object currentValue() {
+        // TODO: implement directly in 2.14 or later, make getCurrentValue() call this
+        return getCurrentValue();
+    }
+
+    /**
+     * Helper method, usually equivalent to:
+     *<code>
+     *   getParsingContext().setCurrentValue(v);
+     *</code>
+     *
+     * @param v Current value to assign for the current input context of this parser
+     *
+     * @since 2.13 (added as replacement for older {@link #setCurrentValue}
+     */
+    public void assignCurrentValue(Object v) {
+        // TODO: implement directly in 2.14 or later, make setCurrentValue() call this
+        setCurrentValue(v);
+    }
+
+    // TODO: deprecate in 2.14 or later
+    /**
+     * Alias for {@link #currentValue()}, to be deprecated in later
+     * Jackson 2.x versions (and removed from Jackson 3.0).
+     *
+     * @return Location of the last processed input unit (byte or character)
+     */
+    public Object getCurrentValue() {
+        JsonStreamContext ctxt = getParsingContext();
+        return (ctxt == null) ? null : ctxt.getCurrentValue();
+    }
+
+    // TODO: deprecate in 2.14 or later
+    /**
+     * Alias for {@link #assignCurrentValue}, to be deprecated in later
+     * Jackson 2.x versions (and removed from Jackson 3.0).
+     *
+     * @param v Current value to assign for the current input context of this parser
+     */
+    public void setCurrentValue(Object v) {
+        JsonStreamContext ctxt = getParsingContext();
+        if (ctxt != null) {
+            ctxt.setCurrentValue(v);
+        }
+    }
+
     /*
     /**********************************************************
     /* Buffer handling
@@ -732,7 +811,7 @@ public abstract class JsonParser
      *    otherwise number of bytes released (0 if there was nothing to release)
      *
      * @throws IOException if write to stream threw exception
-     */    
+     */
     public int releaseBuffered(OutputStream out) throws IOException {
         return -1;
     }
@@ -1168,9 +1247,10 @@ public abstract class JsonParser
         return getCurrentTokenId();
     }
 
+    // TODO: deprecate in 2.14 or later
     /**
      * Alias for {@link #currentToken()}, may be deprecated sometime after
-     * Jackson 2.12 (will be removed from 3.0).
+     * Jackson 2.13 (will be removed from 3.0).
      *
      * @return Type of the token this parser currently points to,
      *   if any: null before any tokens have been read, and
@@ -1178,7 +1258,7 @@ public abstract class JsonParser
     public abstract JsonToken getCurrentToken();
 
     /**
-     * Alias for {@link #currentTokenId()}.
+     * Deprecated alias for {@link #currentTokenId()}.
      *
      * @return {@code int} matching one of constants from {@link JsonTokenId}.
      *
@@ -1350,15 +1430,16 @@ public abstract class JsonParser
      * @param name Name to use as the current name; may be null.
      */
     public abstract void overrideCurrentName(String name);
-    
+
     /*
     /**********************************************************
     /* Public API, access to token information, text
     /**********************************************************
      */
 
+    // TODO: deprecate in 2.14 or later
     /**
-     * See {@link #currentName()}.
+     * Alias of {@link #currentName()}.
      *
      * @return Name of the current field in the parsing context
      *
@@ -2068,7 +2149,7 @@ public abstract class JsonParser
      * JSON Strings map naturally; scalar values get converted to
      * their textual representation.
      * If representation can not be converted to a String value (including structured types
-     * like Objects and Arrays and null token), default value of
+     * like Objects and Arrays and {@code null} token), default value of
      * <b>null</b> will be returned; no exceptions are thrown.
      *
      * @return {@link String} value current token is converted to, if possible; {@code null} otherwise
@@ -2088,7 +2169,7 @@ public abstract class JsonParser
      * JSON Strings map naturally; scalar values get converted to
      * their textual representation.
      * If representation can not be converted to a String value (including structured types
-     * like Objects and Arrays and null token), specified default value
+     * like Objects and Arrays and {@code null} token), specified default value
      * will be returned; no exceptions are thrown.
      *
      * @param def Default value to return if conversion to {@code String} is not possible
@@ -2347,4 +2428,52 @@ public abstract class JsonParser
     protected void _reportUnsupportedOperation() {
         throw new UnsupportedOperationException("Operation not supported by parser of type "+getClass().getName());
     }
+
+    /**
+     * Helper method for constructing {@link JsonParseException}
+     * based on current state of the parser.
+     *
+     * @param msg Base exception message to construct exception with
+     *
+     * @return Read exception (of type {@link JsonParseException}) constructed
+     *
+     * @since 2.13
+     */
+    protected JsonParseException _constructReadException(String msg) {
+        // In 3.0 will be actual `StreamReadException`...
+        return _constructError(msg);
+    }
+
+    protected JsonParseException _constructReadException(String msg, Object arg) {
+        return _constructReadException(String.format(msg, arg));
+    }
+
+    protected JsonParseException _constructReadException(String msg, Object arg1, Object arg2) {
+        return _constructReadException(String.format(msg, arg1, arg2));
+    }
+
+    protected JsonParseException _constructReadException(String msg,
+            Object arg1, Object arg2, Object arg3) {
+        return _constructReadException(String.format(msg, arg1, arg2, arg3));
+    }
+
+    /**
+     * Helper method for constructing {@link JsonParseException}
+     * based on current state of the parser and indicating that the given
+     * {@link Throwable} is the root cause.
+     *
+     * @param msg Base exception message to construct exception with
+     * @param t Root cause to assign
+     *
+     * @return Read exception (of type {@link JsonParseException}) constructed
+     *
+     * @since 2.13
+     */
+    protected JsonParseException _constructReadException(String msg, Throwable t) {
+        JsonParseException e = new JsonParseException(this, msg, t);
+        if (_requestPayload != null) {
+            e = e.withRequestPayload(_requestPayload);
+        }
+        return e;
+    }
 }
