@@ -14,6 +14,8 @@ import static com.fasterxml.jackson.core.JsonTokenId.*;
 
 /**
  * Intermediate base class for non-blocking JSON parsers.
+ *
+ * @since 2.9
  */
 public abstract class NonBlockingJsonParserBase
     extends ParserBase
@@ -99,6 +101,8 @@ public abstract class NonBlockingJsonParserBase
 
     protected final static int MINOR_VALUE_TOKEN_NON_STD = 19;
 
+    protected final static int MINOR_NUMBER_PLUS = 22;
+
     protected final static int MINOR_NUMBER_MINUS = 23;
     protected final static int MINOR_NUMBER_ZERO = 24; // zero as first, possibly trimming multiple
     protected final static int MINOR_NUMBER_MINUSZERO = 25; // "-0" (and possibly more zeroes) receive
@@ -345,7 +349,7 @@ public abstract class NonBlockingJsonParserBase
         int col = _inputPtr - _currInputRowStart + 1; // 1-based
         // Since we track CR and LF separately, max should gives us right answer
         int row = Math.max(_currInputRow, _currInputRowAlt);
-        return new JsonLocation(_getSourceReference(),
+        return new JsonLocation(_contentReference(),
                 _currInputProcessed + (_inputPtr - _currBufferStart), -1L, // bytes, chars
                 row, col);
     }
@@ -353,7 +357,7 @@ public abstract class NonBlockingJsonParserBase
     @Override
     public JsonLocation getTokenLocation()
     {
-        return new JsonLocation(_getSourceReference(),
+        return new JsonLocation(_contentReference(),
                 _tokenInputTotal, -1L, _tokenInputRow, _tokenInputCol);
     }
 
@@ -673,12 +677,10 @@ public abstract class NonBlockingJsonParserBase
         return _addName(quads, 3, lastQuadBytes);
     }
 
-    /**
-     * This is the main workhorse method used when we take a symbol
-     * table miss. It needs to demultiplex individual bytes, decode
-     * multi-byte chars (if any), and then construct Name instance
-     * and add it to the symbol table.
-     */
+    // This is the main workhorse method used when we take a symbol
+    // table miss. It needs to demultiplex individual bytes, decode
+    // multi-byte chars (if any), and then construct Name instance
+    // and add it to the symbol table.
     protected final String _addName(int[] quads, int qlen, int lastQuadBytes) throws JsonParseException
     {
         /* Ok: must decode UTF-8 chars. No other validation is
@@ -787,9 +789,7 @@ public abstract class NonBlockingJsonParserBase
         return _symbols.addName(baseName, quads, qlen);
     }
 
-    /**
-     * Helper method needed to fix [jackson-core#148], masking of 0x00 character
-     */
+    // Helper method needed to fix [jackson-core#148], masking of 0x00 character
     protected final static int _padLastQuad(int q, int bytes) {
         return (bytes == 4) ? q : (q | (-1 << (bytes << 3)));
     }
@@ -799,11 +799,9 @@ public abstract class NonBlockingJsonParserBase
     /* Internal methods, state changes
     /**********************************************************************
      */
-    
-    /**
-     * Helper method called at point when all input has been exhausted and
-     * input feeder has indicated no more input will be forthcoming.
-     */
+
+    // Helper method called at point when all input has been exhausted and
+    // input feeder has indicated no more input will be forthcoming.
     protected final JsonToken _eofAsNextToken() throws IOException {
         _majorState = MAJOR_CLOSED;
         if (!_parsingContext.inRoot()) {
@@ -845,7 +843,7 @@ public abstract class NonBlockingJsonParserBase
         String tokenStr = NON_STD_TOKENS[type];
         _textBuffer.resetWithString(tokenStr);
         if (!isEnabled(Feature.ALLOW_NON_NUMERIC_NUMBERS)) {
-            _reportError("Non-standard token '%s': enable JsonParser.Feature.ALLOW_NON_NUMERIC_NUMBERS to allow",
+            _reportError("Non-standard token '%s': enable `JsonReadFeature.ALLOW_NON_NUMERIC_NUMBERS` to allow",
                     tokenStr);
         }
         _intLength = 0;
