@@ -2,6 +2,7 @@ package com.fasterxml.jackson.core.sym;
 
 import java.util.Arrays;
 import java.util.BitSet;
+import java.util.concurrent.atomic.AtomicReference;
 
 import com.fasterxml.jackson.core.JsonFactory;
 import com.fasterxml.jackson.core.util.InternCache;
@@ -50,7 +51,7 @@ public final class CharsToNameCanonicalizer
      * at least with tests we have.
      */
     public final static int HASH_MULT = 33;
-    
+
     /**
      * Default initial table size. Shouldn't be miniscule (as there's
      * cost to both array realloc and rehashing), but let's keep
@@ -77,20 +78,17 @@ public final class CharsToNameCanonicalizer
     /**
      * Also: to thwart attacks based on hash collisions (which may or may not
      * be cheap to calculate), we will need to detect "too long"
-     * collision chains. Let's start with static value of 255 entries
-     * for the longest legal chain.
+     * collision chains.
+     * Started with static value of 100 entries for the longest legal chain,
+     * but increased in Jackson 2.13 to 150 to work around specific test case.
      *<p>
      * Note: longest chain we have been able to produce without malicious
      * intent has been 38 (with "com.fasterxml.jackson.core.main.TestWithTonsaSymbols");
      * our setting should be reasonable here.
-     *<p>
-     * Also note that value was lowered from 255 (2.3 and earlier) to 100 for 2.4
      * 
-     * @since 2.1
+     * @since 2.1 (changed in 2.13)
      */
-    static final int MAX_COLL_CHAIN_LENGTH = 100;
-
-    final static CharsToNameCanonicalizer sBootstrapSymbolTable = new CharsToNameCanonicalizer();
+    static final int MAX_COLL_CHAIN_LENGTH = 150;
 
     /*
     /**********************************************************
@@ -104,7 +102,15 @@ public final class CharsToNameCanonicalizer
      * defined, and child instance is released (call to <code>release</code>),
      * parent's shared tables may be updated from the child instance.
      */
-    private CharsToNameCanonicalizer _parent;
+    final protected CharsToNameCanonicalizer _parent;
+
+    /**
+     * Member that is only used by the root table instance: root
+     * passes immutable state info child instances, and children
+     * may return new state if they add entries to the table.
+     * Child tables do NOT use the reference.
+     */
+    final protected AtomicReference<TableInfo> _tableInfo;
 
     /**
      * Seed value we use as the base to make hash codes non-static between
@@ -115,16 +121,18 @@ public final class CharsToNameCanonicalizer
      * 
      * @since 2.1
      */
-    final private int _hashSeed;
+    final protected int _seed;
+
+    final protected int _flags;
 
-    final private int _flags;
-    
     /**
      * Whether any canonicalization should be attempted (whether using
-     * intern or not)
+     * intern or not.
+     *<p>
+     * NOTE: non-final since we may need to disable this with overflow.
      */
-    private boolean _canonicalize;
-    
+    protected boolean _canonicalize;
+
     /*
     /**********************************************************
     /* Actual symbol table data
@@ -135,7 +143,7 @@ public final class CharsToNameCanonicalizer
      * Primary matching symbols; it's expected most match occur from
      * here.
      */
-    private String[] _symbols;
+    protected String[] _symbols;
 
     /**
      * Overflow buckets; if primary doesn't match, lookup is done
@@ -144,27 +152,27 @@ public final class CharsToNameCanonicalizer
      * Note: Number of buckets is half of number of symbol entries, on
      * assumption there's less need for buckets.
      */
-    private Bucket[] _buckets;
+    protected Bucket[] _buckets;
 
     /**
      * Current size (number of entries); needed to know if and when
      * rehash.
      */
-    private int _size;
+    protected int _size;
 
     /**
      * Limit that indicates maximum size this instance can hold before
      * it needs to be expanded and rehashed. Calculated using fill
      * factor passed in to constructor.
      */
-    private int _sizeThreshold;
+    protected int _sizeThreshold;
 
     /**
      * Mask used to get index from hash values; equal to
      * <code>_buckets.length - 1</code>, when _buckets.length is
      * a power of two.
      */
-    private int _indexMask;
+    protected int _indexMask;
 
     /**
      * We need to keep track of the longest collision list; this is needed
@@ -173,7 +181,7 @@ public final class CharsToNameCanonicalizer
      * 
      * @since 2.1
      */
-    private int _longestCollisionList;
+    protected int _longestCollisionList;
 
     /*
     /**********************************************************
@@ -182,12 +190,17 @@ public final class CharsToNameCanonicalizer
      */
 
     /**
-     * Flag that indicates if any changes have been made to the data;
-     * used to both determine if bucket array needs to be copied when
-     * (first) change is made, and potentially if updated bucket list
-     * is to be resync'ed back to master instance.
+     * Flag that indicates whether underlying data structures for
+     * the main hash area are shared or not. If they are, then they
+     * need to be handled in copy-on-write way, i.e. if they need
+     * to be modified, a copy needs to be made first; at this point
+     * it will not be shared any more, and can be modified.
+     *<p>
+     * This flag needs to be checked both when adding new main entries,
+     * and when adding new collision list queues (i.e. creating a new
+     * collision list head entry)
      */
-    private boolean _dirty;
+    protected boolean _hashShared;
 
     /*
     /**********************************************************
@@ -203,11 +216,67 @@ public final class CharsToNameCanonicalizer
      * 
      * @since 2.4
      */
-    private BitSet _overflows;
+    protected BitSet _overflows;
 
     /*
     /**********************************************************
-    /* Life-cycle
+    /* Life-cycle: constructors
+    /**********************************************************
+     */
+
+    /**
+     * Main method for constructing a root symbol table instance.
+     */
+    private CharsToNameCanonicalizer(int seed)
+    {
+        _parent = null;
+        _seed = seed;
+        
+        // these settings don't really matter for the bootstrap instance
+        _canonicalize = true;
+        _flags = -1;
+        // And we'll also set flags so no copying of buckets is needed:
+        _hashShared = false; // doesn't really matter for root instance
+        _longestCollisionList = 0;
+
+        _tableInfo = new AtomicReference<TableInfo>(TableInfo.createInitial(DEFAULT_T_SIZE));
+        // and actually do NOT assign buffers so we'll find if anyone tried to
+        // use root instance
+    }
+
+    /**
+     * Internal constructor used when creating child instances.
+     */
+    private CharsToNameCanonicalizer(CharsToNameCanonicalizer parent, int flags, int seed,
+            TableInfo parentState)
+    {
+        _parent = parent;
+        _seed = seed;
+        _tableInfo = null; // not used by child tables
+        _flags = flags;
+        _canonicalize = JsonFactory.Feature.CANONICALIZE_FIELD_NAMES.enabledIn(flags);
+
+        // Then copy shared state
+        _symbols = parentState.symbols;
+        _buckets = parentState.buckets;
+
+        _size = parentState.size;
+        _longestCollisionList = parentState.longestCollisionList;
+
+        // Hard-coded fill factor, 75%
+        int arrayLen = (_symbols.length);
+        _sizeThreshold = _thresholdSize(arrayLen);
+        _indexMask =  (arrayLen - 1);
+
+        // Need to make copies of arrays, if/when adding new entries
+        _hashShared = true;
+    }
+
+    private static int _thresholdSize(int hashAreaSize) { return hashAreaSize - (hashAreaSize >> 2); }
+
+    /*
+    /**********************************************************
+    /* Life-cycle: factory methods, merging
     /**********************************************************
      */
 
@@ -215,73 +284,22 @@ public final class CharsToNameCanonicalizer
      * Method called to create root canonicalizer for a {@link com.fasterxml.jackson.core.JsonFactory}
      * instance. Root instance is never used directly; its main use is for
      * storing and sharing underlying symbol arrays as needed.
+     *
+     * @return Root instance to use for constructing new child instances 
      */
     public static CharsToNameCanonicalizer createRoot() {
-        /* [Issue-21]: Need to use a variable seed, to thwart hash-collision
-         * based attacks.
-         */
+        // Need to use a variable seed, to thwart hash-collision based attacks.
+        // 14-Feb-2017, tatu: not sure it actually helps, at all, since it won't
+        //   change mixing or any of the steps. Should likely just remove in future.
         long now = System.currentTimeMillis();
         // ensure it's not 0; and might as well require to be odd so:
         int seed = (((int) now) + ((int) (now >>> 32))) | 1;
         return createRoot(seed);
     }
-    
-    protected static CharsToNameCanonicalizer createRoot(int hashSeed) {
-        return sBootstrapSymbolTable.makeOrphan(hashSeed);
+
+    protected static CharsToNameCanonicalizer createRoot(int seed) {
+        return new CharsToNameCanonicalizer(seed);
     }
-
-    /**
-     * Main method for constructing a master symbol table instance.
-     */
-    private CharsToNameCanonicalizer() {
-        // these settings don't really matter for the bootstrap instance
-        _canonicalize = true;
-        _flags = -1;
-        // And we'll also set flags so no copying of buckets is needed:
-        _dirty = true;
-        _hashSeed = 0;
-        _longestCollisionList = 0;
-        initTables(DEFAULT_T_SIZE);
-    }
-
-    private void initTables(int initialSize)
-    {
-        _symbols = new String[initialSize];
-        _buckets = new Bucket[initialSize >> 1];
-        // Mask is easy to calc for powers of two.
-        _indexMask = initialSize - 1;
-        _size = 0;
-        _longestCollisionList = 0;
-        // Hard-coded fill factor is 75%
-        _sizeThreshold = _thresholdSize(initialSize);
-    }
-
-    private static int _thresholdSize(int hashAreaSize) { return hashAreaSize - (hashAreaSize >> 2); }
-    
-    /**
-     * Internal constructor used when creating child instances.
-     */
-    private CharsToNameCanonicalizer(CharsToNameCanonicalizer parent, int flags,
-            String[] symbols, Bucket[] buckets, int size, int hashSeed, int longestColl) {
-        _parent = parent;
-
-        _flags = flags;
-        _canonicalize = JsonFactory.Feature.CANONICALIZE_FIELD_NAMES.enabledIn(flags);
-
-        _symbols = symbols;
-        _buckets = buckets;
-        _size = size;
-        _hashSeed = hashSeed;
-        // Hard-coded fill factor, 75%
-        int arrayLen = (symbols.length);
-        _sizeThreshold = _thresholdSize(arrayLen);
-        _indexMask =  (arrayLen - 1);
-        _longestCollisionList = longestColl;
-
-        // Need to make copies of arrays, if/when adding new entries
-        _dirty = false;
-    }
-
     /**
      * "Factory" method; will create a new child instance of this symbol
      * table. It will be a copy-on-write instance, ie. it will only use
@@ -293,31 +311,32 @@ public final class CharsToNameCanonicalizer
      * actively. Instead, a separate 'root' instance should be used
      * on which only makeChild/mergeChild are called, but instance itself
      * is not used as a symbol table.
+     *
+     * @param flags Bit flags of active {@link com.fasterxml.jackson.core.JsonFactory.Feature}s enabled.
+     *
+     * @return Actual canonicalizer instance that can be used by a parser
      */
     public CharsToNameCanonicalizer makeChild(int flags) {
-        /* 24-Jul-2012, tatu: Trying to reduce scope of synchronization, assuming
-         *   that synchronizing construction is the (potentially) expensive part,
-         *   and not so much short copy-the-variables thing.
-         */
-        final String[] symbols;
-        final Bucket[] buckets;
-        final int size;
-        final int hashSeed;
-        final int longestCollisionList;
-        
-        synchronized (this) {
-            symbols = _symbols;
-            buckets = _buckets;
-            size = _size;
-            hashSeed = _hashSeed;
-            longestCollisionList = _longestCollisionList;
-        }
-        return new CharsToNameCanonicalizer(this, flags,
-                symbols, buckets, size, hashSeed, longestCollisionList);
+        return new CharsToNameCanonicalizer(this, flags, _seed, _tableInfo.get());
     }
 
-    private CharsToNameCanonicalizer makeOrphan(int seed) {
-        return new CharsToNameCanonicalizer(null, -1, _symbols, _buckets, _size, seed, _longestCollisionList);
+    /**
+     * Method called by the using code to indicate it is done with this instance.
+     * This lets instance merge accumulated changes into parent (if need be),
+     * safely and efficiently, and without calling code having to know about parent
+     * information.
+     */
+    public void release() {
+        // If nothing has been added, nothing to do
+        if (!maybeDirty()) { return; }
+
+        // we will try to merge if child table has new entries
+        if (_parent != null && _canonicalize) { // canonicalize set to false if max size was reached
+            _parent.mergeChild(new TableInfo(this));
+            // Let's also mark this instance as dirty, so that just in
+            // case release was too early, there's no corruption of possibly shared data.
+            _hashShared = true;
+        }
     }
 
     /**
@@ -327,55 +346,25 @@ public final class CharsToNameCanonicalizer
      * Note that caller has to make sure symbol table passed in is
      * really a child or sibling of this symbol table.
      */
-    private void mergeChild(CharsToNameCanonicalizer child) {
-        /* One caveat: let's try to avoid problems with
-         * degenerate cases of documents with generated "random"
-         * names: for these, symbol tables would bloat indefinitely.
-         * One way to do this is to just purge tables if they grow
-         * too large, and that's what we'll do here.
-         */
-        if (child.size() > MAX_ENTRIES_FOR_REUSE) {
-            // Should there be a way to get notified about this event, to log it or such?
-            // (as it's somewhat abnormal thing to happen)
-            // At any rate, need to clean up the tables, then:
-            synchronized (this) {
-                initTables(DEFAULT_T_SIZE * 4); // no point in starting from tiny tho
-                // Dirty flag... well, let's just clear it. Shouldn't really matter for master tables
-                // (which this is, given something is merged to it)
-                _dirty = false;
-            }
-        } else {
-            // Otherwise, we'll merge changed stuff in, if there are  more entries (which
-            // may not be the case if one of siblings has added symbols first or such)
-            if (child.size() <= size()) { // nothing to add
-                return;
-            }
-            // Okie dokie, let's get the data in!
-            synchronized (this) {
-                _symbols = child._symbols;
-                _buckets = child._buckets;
-                _size = child._size;
-                _sizeThreshold = child._sizeThreshold;
-                _indexMask = child._indexMask;
-                _longestCollisionList = child._longestCollisionList;
-                // Dirty flag... well, let's just clear it. Shouldn't really matter for master tables
-                // (which this is, given something is merged to it)
-                _dirty = false;
-            }
-        }
-    }
+    private void mergeChild(TableInfo childState)
+    {
+        final int childCount = childState.size;
+        TableInfo currState = _tableInfo.get();
 
-    public void release(){
-        // If nothing has been added, nothing to do
-        if (!maybeDirty()) { return; }
-        if (_parent != null && _canonicalize) { // canonicalize set to false if max size was reached
-            _parent.mergeChild(this);
-            /* Let's also mark this instance as dirty, so that just in
-             * case release was too early, there's no corruption
-             * of possibly shared data.
-             */
-            _dirty = false;
+        // Should usually grow; but occasionally could also shrink if (but only if)
+        // collision list overflow ends up clearing some collision lists.
+        if (childCount == currState.size) {
+            return;
         }
+        // One caveat: let's try to avoid problems with  degenerate cases of documents with
+        // generated "random" names: for these, symbol tables would bloat indefinitely.
+        // One way to do this is to just purge tables if they grow
+        // too large, and that's what we'll do here.
+        if (childCount > MAX_ENTRIES_FOR_REUSE) {
+            // At any rate, need to clean up the tables
+            childState = TableInfo.createInitial(DEFAULT_T_SIZE);
+        }
+        _tableInfo.compareAndSet(currState, childState);
     }
 
     /*
@@ -384,24 +373,35 @@ public final class CharsToNameCanonicalizer
     /**********************************************************
      */
 
-    public int size() { return _size; }
+    /**
+     * @return Number of symbol entries contained by this canonicalizer instance
+     */
+    public int size() {
+        if (_tableInfo != null) { // root table
+            return _tableInfo.get().size;
+        }
+        // nope, child table
+        return _size;
+    }
 
     /**
      * Method for checking number of primary hash buckets this symbol
      * table uses.
      * 
-     * @since 2.1
+     * @return number of primary slots table has currently
      */
     public int bucketCount() {  return _symbols.length; }
-    public boolean maybeDirty() { return _dirty; }
-    public int hashSeed() { return _hashSeed; }
-    
+    public boolean maybeDirty() { return !_hashShared; }
+    public int hashSeed() { return _seed; }
+
     /**
      * Method mostly needed by unit tests; calculates number of
      * entries that are in collision list. Value can be at most
      * ({@link #size} - 1), but should usually be much lower, ideally 0.
      * 
      * @since 2.1
+     *
+     * @return Number of collisions in the primary hash area
      */
     public int collisionCount() {
         int count = 0;
@@ -418,7 +418,9 @@ public final class CharsToNameCanonicalizer
      * Method mostly needed by unit tests; calculates length of the
      * longest collision chain. This should typically be a low number,
      * but may be up to {@link #size} - 1 in the pathological case
-     * 
+     *
+     * @return Length of the collision chain
+     *
      * @since 2.1
      */
     public int maxCollisionLength() { return _longestCollisionList; }
@@ -483,17 +485,16 @@ public final class CharsToNameCanonicalizer
         }
         return null;
     }
-    
+
     private String _addSymbol(char[] buffer, int start, int len, int h, int index)
     {
-        if (!_dirty) { //need to do copy-on-write?
+        if (_hashShared) { //need to do copy-on-write?
             copyArrays();
-            _dirty = true;
+            _hashShared = false;
         } else if (_size >= _sizeThreshold) { // Need to expand?
             rehash();
-            /* Need to recalc hash; rare occurence (index mask has been
-             * recalculated as part of rehash)
-             */
+            // Need to recalc hash; rare occurrence (index mask has been
+             // recalculated as part of rehash)
             index = _hashToIndex(calcHash(buffer, start, len));
         }
 
@@ -510,40 +511,47 @@ public final class CharsToNameCanonicalizer
             Bucket newB = new Bucket(newSymbol, _buckets[bix]);
             int collLen = newB.length;
             if (collLen > MAX_COLL_CHAIN_LENGTH) {
-                /* 23-May-2014, tatu: Instead of throwing an exception right away, let's handle
-                 *   in bit smarter way.
-                 */
-                _handleSpillOverflow(bix, newB);
+                // 23-May-2014, tatu: Instead of throwing an exception right away,
+                //    let's handle in bit smarter way.
+                _handleSpillOverflow(bix, newB, index);
             } else {
                 _buckets[bix] = newB;
                 _longestCollisionList = Math.max(collLen, _longestCollisionList);
             }
         }
-
         return newSymbol;
     }
 
-    private void _handleSpillOverflow(int bindex, Bucket newBucket)
+    /**
+     * Method called when an overflow bucket has hit the maximum expected length:
+     * this may be a case of DoS attack. Deal with it based on settings by either
+     * clearing up bucket (to avoid indefinite expansion) or throwing exception.
+     * Currently the first overflow for any single bucket DOES NOT throw an exception,
+     * only second time (per symbol table instance)
+     */
+    private void _handleSpillOverflow(int bucketIndex, Bucket newBucket, int mainIndex)
     {
         if (_overflows == null) {
             _overflows = new BitSet();
-            _overflows.set(bindex);
+            _overflows.set(bucketIndex);
         } else {
-            if (_overflows.get(bindex)) {
-                // Has happened once already, so not a coincident...
+            if (_overflows.get(bucketIndex)) {
+                // Has happened once already for this bucket index, so probably not coincidental...
                 if (JsonFactory.Feature.FAIL_ON_SYMBOL_HASH_OVERFLOW.enabledIn(_flags)) {
-                    reportTooManyCollisions(MAX_COLL_CHAIN_LENGTH);
+                    _reportTooManyCollisions(MAX_COLL_CHAIN_LENGTH);
                 }
-                // but even if we don't fail, we will stop canonicalizing:
+                // but even if we don't fail, we will stop canonicalizing as safety measure
+                // (so as not to cause problems with PermGen)
                 _canonicalize = false;
             } else {
-                _overflows.set(bindex);
+                _overflows.set(bucketIndex);
             }
         }
+
         // regardless, if we get this far, clear up the bucket, adjust size appropriately.
-        _symbols[bindex + bindex] = newBucket.symbol;
-        _buckets[bindex] = null;
-        // newBucket contains new symbol; but we wil 
+        _symbols[mainIndex] = newBucket.symbol;
+        _buckets[bucketIndex] = null;
+        // newBucket contains new symbol; but we will
         _size -= (newBucket.length);
         // we could calculate longest; but for now just mark as invalid
         _longestCollisionList = -1;
@@ -552,6 +560,10 @@ public final class CharsToNameCanonicalizer
     /**
      * Helper method that takes in a "raw" hash value, shuffles it as necessary,
      * and truncates to be used as the index.
+     *
+     * @param rawHash Raw hash value to use for calculating index
+     *
+     * @return Index value calculated
      */
     public int _hashToIndex(int rawHash) {
         // doing these seems to help a bit
@@ -560,18 +572,21 @@ public final class CharsToNameCanonicalizer
         rawHash += (rawHash >>> 3);
         return (rawHash & _indexMask);
     }
-    
+
     /**
      * Implementation of a hashing method for variable length
      * Strings. Most of the time intention is that this calculation
      * is done by caller during parsing, not here; however, sometimes
      * it needs to be done for parsed "String" too.
      *
-     * @param len Length of String; has to be at least 1 (caller guarantees
-     *   this pre-condition)
+     * @param buffer Input buffer that contains name to decode
+     * @param start Pointer to the first character of the name
+     * @param len Length of String; has to be at least 1 (caller guarantees)
+     *
+     * @return Hash code calculated
      */
     public int calcHash(char[] buffer, int start, int len) {
-        int hash = _hashSeed;
+        int hash = _seed;
         for (int i = start, end = start+len; i < end; ++i) {
             hash = (hash * HASH_MULT) + (int) buffer[i];
         }
@@ -583,7 +598,7 @@ public final class CharsToNameCanonicalizer
     {
         final int len = key.length();
         
-        int hash = _hashSeed;
+        int hash = _seed;
         for (int i = 0; i < len; ++i) {
             hash = (hash * HASH_MULT) + (int) key.charAt(i);
         }
@@ -616,7 +631,7 @@ public final class CharsToNameCanonicalizer
      * entries.
      */
     private void rehash() {
-        int size = _symbols.length;
+        final int size = _symbols.length;
         int newSize = size + size;
 
         /* 12-Mar-2010, tatu: Let's actually limit maximum size we are
@@ -624,32 +639,30 @@ public final class CharsToNameCanonicalizer
          *    name sets (unique [non-repeating] names)
          */
         if (newSize > MAX_T_SIZE) {
-            /* If this happens, there's no point in either growing or shrinking hash areas.
-             * Rather, let's just cut our losses and stop canonicalizing.
-             */
+            // If this happens, there's no point in either growing or shrinking hash areas.
+            // Rather, let's just cut our losses and stop canonicalizing.
             _size = 0;
             _canonicalize = false;
             // in theory, could just leave these as null, but...
             _symbols = new String[DEFAULT_T_SIZE];
             _buckets = new Bucket[DEFAULT_T_SIZE>>1];
             _indexMask = DEFAULT_T_SIZE-1;
-            _dirty = true;
+            _hashShared = false;
             return;
         }
-        
-        String[] oldSyms = _symbols;
-        Bucket[] oldBuckets = _buckets;
+
+        final String[] oldSyms = _symbols;
+        final Bucket[] oldBuckets = _buckets;
         _symbols = new String[newSize];
         _buckets = new Bucket[newSize >> 1];
         // Let's update index mask, threshold, now (needed for rehashing)
         _indexMask = newSize - 1;
         _sizeThreshold = _thresholdSize(newSize);
-        
+
         int count = 0; // let's do sanity check
 
-        /* Need to do two loops, unfortunately, since spill-over area is
-         * only half the size:
-         */
+        // Need to do two loops, unfortunately, since spill-over area is
+        // only half the size:
         int maxColl = 0;
         for (int i = 0; i < size; ++i) {
             String symbol = oldSyms[i];
@@ -667,8 +680,8 @@ public final class CharsToNameCanonicalizer
             }
         }
 
-        size >>= 1;
-        for (int i = 0; i < size; ++i) {
+        final int bucketSize = (size >> 1);
+        for (int i = 0; i < bucketSize; ++i) {
             Bucket b = oldBuckets[i];
             while (b != null) {
                 ++count;
@@ -689,18 +702,52 @@ public final class CharsToNameCanonicalizer
         _overflows = null;
 
         if (count != _size) {
-            throw new Error("Internal error on SymbolTable.rehash(): had "+_size+" entries; now have "+count+".");
+            throw new IllegalStateException(String.format(
+                    "Internal error on SymbolTable.rehash(): had %d entries; now have %d",
+                    _size, count));
         }
     }
 
     /**
+     * @param maxLen Maximum allowed length of collision chain
+     *
      * @since 2.1
      */
-    protected void reportTooManyCollisions(int maxLen) {
+    protected void _reportTooManyCollisions(int maxLen) {
         throw new IllegalStateException("Longest collision chain in symbol table (of size "+_size
                 +") now exceeds maximum, "+maxLen+" -- suspect a DoS attack based on hash collisions");
     }
 
+    // since 2.10, for tests only
+    /**
+     * Diagnostics method that will verify that internal data structures are consistent;
+     * not meant as user-facing method but only for test suites and possible troubleshooting.
+     *
+     * @since 2.10
+     */
+    protected void verifyInternalConsistency() {
+        int count = 0;
+        final int size = _symbols.length;
+
+        for (int i = 0; i < size; ++i) {
+            String symbol = _symbols[i];
+            if (symbol != null) {
+                ++count;
+            }
+        }
+
+        final int bucketSize = (size >> 1);
+        for (int i = 0; i < bucketSize; ++i) {
+            for (Bucket b = _buckets[i]; b != null; b = b.next) {
+                ++count;
+            }
+        }
+        if (count != _size) {
+            throw new IllegalStateException(String.format("Internal error: expected internal size %d vs calculated count %d",
+                    _size, count));
+        }
+    }
+
     // For debugging, comment out
     /*
     @Override
@@ -752,7 +799,7 @@ public final class CharsToNameCanonicalizer
 
     /*
     /**********************************************************
-    /* Bucket class
+    /* Helper classes
     /**********************************************************
      */
 
@@ -785,4 +832,42 @@ public final class CharsToNameCanonicalizer
             return symbol;
         }
     }
+
+    /**
+     * Immutable value class used for sharing information as efficiently
+     * as possible, by only require synchronization of reference manipulation
+     * but not access to contents.
+     * 
+     * @since 2.8.7
+     */
+    private final static class TableInfo
+    {
+        final int size;
+        final int longestCollisionList;
+        final String[] symbols;
+        final Bucket[] buckets;
+
+        public TableInfo(int size, int longestCollisionList,
+                String[] symbols, Bucket[] buckets)
+        {
+            this.size = size;
+            this.longestCollisionList = longestCollisionList;
+            this.symbols = symbols;
+            this.buckets = buckets;
+        }
+
+        public TableInfo(CharsToNameCanonicalizer src)
+        {
+            this.size = src._size;
+            this.longestCollisionList = src._longestCollisionList;
+            this.symbols = src._symbols;
+            this.buckets = src._buckets;
+        }
+
+        public static TableInfo createInitial(int sz) {
+            return new TableInfo(0,
+                    0, // longestCollisionList
+                    new String[sz], new Bucket[sz >> 1]);
+        }
+    }
 }
