@@ -7,7 +7,7 @@ import java.util.*;
 import com.fasterxml.jackson.core.io.NumberInput;
 
 /**
- * TextBuffer is a class similar to {@link StringBuffer}, with
+ * TextBuffer is a class similar to {@link java.lang.StringBuffer}, with
  * following differences:
  *<ul>
  *  <li>TextBuffer uses segments character arrays, to avoid having
@@ -30,14 +30,16 @@ public final class TextBuffer
 
     /**
      * Let's start with sizable but not huge buffer, will grow as necessary
+     *<p>
+     * Reduced from 1000 down to 500 in 2.10.
      */
-    final static int MIN_SEGMENT_LEN = 1000;
-    
+    final static int MIN_SEGMENT_LEN = 500;
+
     /**
-     * Let's limit maximum segment length to something sensible
-     * like 256k
+     * Let's limit maximum segment length to something sensible.
+     * For 2.10, let's limit to using 64kc chunks (128 kB) -- was 256kC/512kB up to 2.9
      */
-    final static int MAX_SEGMENT_LEN = 0x40000;
+    final static int MAX_SEGMENT_LEN = 0x10000;
     
     /*
     /**********************************************************
@@ -87,7 +89,7 @@ public final class TextBuffer
     // // // Currently used segment; not (yet) contained in _seqments
 
     /**
-     * Amount of characters in segments in {@link _segments}
+     * Amount of characters in segments in {@link #_segments}
      */
     private int _segmentSize;
 
@@ -122,23 +124,60 @@ public final class TextBuffer
         _allocator = allocator;
     }
 
+    // @since 2.10
+    protected TextBuffer(BufferRecycler allocator, char[] initialSegment) {
+        _allocator = allocator;
+        _currentSegment = initialSegment;
+        _currentSize = initialSegment.length;
+        _inputStart = -1;
+    }
+
+    /**
+     * Factory method for constructing an instance with no allocator, and
+     * with initial full segment.
+     *
+     * @param initialSegment Initial, full segment to use for creating buffer (buffer
+     *   {@link #size()} would return length of {@code initialSegment})
+     *
+     * @return TextBuffer constructed
+     *
+     * @since 2.10
+     */
+    public static TextBuffer fromInitial(char[] initialSegment) {
+        return new TextBuffer(null, initialSegment);
+    }
+
     /**
      * Method called to indicate that the underlying buffers should now
      * be recycled if they haven't yet been recycled. Although caller
      * can still use this text buffer, it is not advisable to call this
      * method if that is likely, since next time a buffer is needed,
      * buffers need to reallocated.
-     * Note: calling this method automatically also clears contents
-     * of the buffer.
+     *<p>
+     * Note: since Jackson 2.11, calling this method will NOT clear already
+     * aggregated contents (that is, {@code _currentSegment}, to retain
+     * current token text if (but only if!) already aggregated.
      */
     public void releaseBuffers()
     {
-        if (_allocator == null) {
-            resetWithEmpty();
-        } else {
+        // inlined `resetWithEmpty()` (except leaving `_resultString` as-is
+        {
+            _inputStart = -1;
+            _currentSize = 0;
+            _inputLen = 0;
+
+            _inputBuffer = null;
+            // note: _resultString retained (see https://github.com/FasterXML/jackson-databind/issues/2635
+            // for reason)
+            _resultArray = null; // should this be retained too?
+
+            if (_hasSegments) {
+                clearSegments();
+            }
+        }
+
+        if (_allocator != null) {
             if (_currentSegment != null) {
-                // First, let's get rid of all but the largest char array
-                resetWithEmpty();
                 // And then return that array
                 char[] buf = _currentSegment;
                 _currentSegment = null;
@@ -167,13 +206,42 @@ public final class TextBuffer
         }
     }
 
+    /**
+     * Method for clearing out possibly existing content, and replacing them with
+     * a single-character content (so {@link #size()} would return {@code 1})
+     *
+     * @param ch Character to set as the buffer contents
+     *
+     * @since 2.9
+     */
+    public void resetWith(char ch)
+    {
+        _inputStart = -1;
+        _inputLen = 0;
+
+        _resultString = null;
+        _resultArray = null;
+
+        if (_hasSegments) {
+            clearSegments();
+        } else if (_currentSegment == null) {
+            _currentSegment = buf(1);
+        }
+        _currentSegment[0] = ch; // lgtm [java/dereferenced-value-may-be-null]
+        _currentSize = _segmentSize = 1;
+    }
+    
     /**
      * Method called to initialize the buffer with a shared copy of data;
      * this means that buffer will just have pointers to actual data. It
      * also means that if anything is to be appended to the buffer, it
      * will first have to unshare it (make a local copy).
+     *
+     * @param buf Buffer that contains shared contents
+     * @param offset Offset of the first content character in {@code buf}
+     * @param len Length of content in {@code buf}
      */
-    public void resetWithShared(char[] buf, int start, int len)
+    public void resetWithShared(char[] buf, int offset, int len)
     {
         // First, let's clear intermediate values, if any:
         _resultString = null;
@@ -181,7 +249,7 @@ public final class TextBuffer
 
         // Then let's mark things we need about input buffer
         _inputBuffer = buf;
-        _inputStart = start;
+        _inputStart = offset;
         _inputLen = len;
 
         // And then reset internal input buffers, if necessary:
@@ -190,7 +258,7 @@ public final class TextBuffer
         }
     }
 
-    public void resetWithCopy(char[] buf, int start, int len)
+    public void resetWithCopy(char[] buf, int offset, int len)
     {
         _inputBuffer = null;
         _inputStart = -1; // indicates shared buffer not used
@@ -206,7 +274,26 @@ public final class TextBuffer
             _currentSegment = buf(len);
         }
         _currentSize = _segmentSize = 0;
-        append(buf, start, len);
+        append(buf, offset, len);
+    }
+
+    // @since 2.9
+    public void resetWithCopy(String text, int start, int len)
+    {
+        _inputBuffer = null;
+        _inputStart = -1;
+        _inputLen = 0;
+
+        _resultString = null;
+        _resultArray = null;
+
+        if (_hasSegments) {
+            clearSegments();
+        } else if (_currentSegment == null) {
+            _currentSegment = buf(len);
+        }
+        _currentSize = _segmentSize = 0;
+        append(text, start, len);
     }
 
     public void resetWithString(String value)
@@ -224,11 +311,21 @@ public final class TextBuffer
         _currentSize = 0;
         
     }
-    
+
     /**
-     * Helper method used to find a buffer to use, ideally one
-     * recycled earlier.
+     * Method for accessing the currently active (last) content segment
+     * without changing state of the buffer
+     *
+     * @return Currently active (last) content segment
+     *
+     * @since 2.9
      */
+    public char[] getBufferWithoutReset() {
+        return _currentSegment;
+    }
+
+    // Helper method used to find a buffer to use, ideally one
+    // recycled earlier.
     private char[] buf(int needed)
     {
         if (_allocator != null) {
@@ -258,7 +355,7 @@ public final class TextBuffer
      */
 
     /**
-     * @return Number of characters currently stored by this collector
+     * @return Number of characters currently stored in this buffer
      */
     public int size() {
         if (_inputStart >= 0) { // shared copy from input buf
@@ -285,6 +382,9 @@ public final class TextBuffer
     /**
      * Method that can be used to check whether textual contents can
      * be efficiently accessed using {@link #getTextBuffer}.
+     *
+     * @return {@code True} if access via {@link #getTextBuffer()} would be efficient
+     *   (that is, content already available as aggregated {@code char[]})
      */
     public boolean hasTextAsCharacters()
     {
@@ -296,9 +396,11 @@ public final class TextBuffer
     }
 
     /**
-     * Accessor that may be used to get the contents of this buffer in a single
-     * <code>char</code> array regardless of whether they were collected in a segmented
-     * fashion or not.
+     * Accessor that may be used to get the contents of this buffer as a single
+     * {@code char[]} regardless of whether they were collected in a segmented
+     * fashion or not: this typically require allocation of the result buffer.
+     *
+     * @return Aggregated {@code char[]} that contains all buffered content
      */
     public char[] getTextBuffer()
     {
@@ -322,6 +424,13 @@ public final class TextBuffer
     /**********************************************************
      */
 
+    /**
+     * Accessor that may be used to get the contents of this buffer as a single
+     * {@code String} regardless of whether they were collected in a segmented
+     * fashion or not: this typically require construction of the result String.
+     *
+     * @return Aggregated buffered contents as a {@link java.lang.String}
+     */
     public String contentsAsString()
     {
         if (_resultString == null) {
@@ -358,6 +467,7 @@ public final class TextBuffer
                 }
             }
         }
+
         return _resultString;
     }
  
@@ -372,6 +482,10 @@ public final class TextBuffer
     /**
      * Convenience method for converting contents of the buffer
      * into a {@link BigDecimal}.
+     *
+     * @return Buffered text value parsed as a {@link BigDecimal}, if possible
+     *
+     * @throws NumberFormatException if contents are not a valid Java number
      */
     public BigDecimal contentsAsDecimal() throws NumberFormatException
     {
@@ -394,12 +508,125 @@ public final class TextBuffer
     /**
      * Convenience method for converting contents of the buffer
      * into a Double value.
+     *
+     * @return Buffered text value parsed as a {@link Double}, if possible
+     *
+     * @throws NumberFormatException if contents are not a valid Java number
+     * @deprecated use {@link #contentsAsDouble(boolean)}
      */
+    @Deprecated
     public double contentsAsDouble() throws NumberFormatException {
-        return NumberInput.parseDouble(contentsAsString());
+        return contentsAsDouble(false);
     }
 
     /**
+     * Convenience method for converting contents of the buffer
+     * into a Double value.
+     *
+     * @param useFastParser whether to use {@link com.fasterxml.jackson.core.io.doubleparser}
+     * @return Buffered text value parsed as a {@link Double}, if possible
+     *
+     * @throws NumberFormatException if contents are not a valid Java number
+     * @since 2.14
+     */
+    public double contentsAsDouble(final boolean useFastParser) throws NumberFormatException {
+        return NumberInput.parseDouble(contentsAsString(), useFastParser);
+    }
+
+    /**
+     * Convenience method for converting contents of the buffer
+     * into a Float value.
+     *
+     * @return Buffered text value parsed as a {@link Float}, if possible
+     *
+     * @throws NumberFormatException if contents are not a valid Java number
+     * @since 2.14
+     * @deprecated use {@link #contentsAsFloat(boolean)}
+     */
+    @Deprecated
+    public float contentsAsFloat() throws NumberFormatException {
+        return contentsAsFloat(false);
+    }
+
+    /**
+     * Convenience method for converting contents of the buffer
+     * into a Float value.
+     *
+     * @param useFastParser whether to use {@link com.fasterxml.jackson.core.io.doubleparser}
+     * @return Buffered text value parsed as a {@link Float}, if possible
+     *
+     * @throws NumberFormatException if contents are not a valid Java number
+     * @since 2.14
+     */
+    public float contentsAsFloat(final boolean useFastParser) throws NumberFormatException {
+        return NumberInput.parseFloat(contentsAsString(), useFastParser);
+    }
+
+    /**
+     * Specialized convenience method that will decode a 32-bit int,
+     * of at most 9 digits (and possible leading minus sign).
+     *<p>
+     * NOTE: method DOES NOT verify that the contents actually are a valid
+     * Java {@code int} value (either regarding format, or value range): caller
+     * MUST validate that.
+     *
+     * @param neg Whether contents start with a minus sign
+     *
+     * @return Buffered text value parsed as an {@code int} using
+     *   {@link NumberInput#parseInt(String)} method (which does NOT validate input)
+     *
+     * @since 2.9
+     */
+    public int contentsAsInt(boolean neg) {
+        if ((_inputStart >= 0) && (_inputBuffer != null)) {
+            if (neg) {
+                return -NumberInput.parseInt(_inputBuffer, _inputStart+1, _inputLen-1);
+            }
+            return NumberInput.parseInt(_inputBuffer, _inputStart, _inputLen);
+        }
+        if (neg) {
+            return -NumberInput.parseInt(_currentSegment, 1, _currentSize-1);
+        }
+        return NumberInput.parseInt(_currentSegment, 0, _currentSize);
+    }
+
+    /**
+     * Specialized convenience method that will decode a 64-bit int,
+     * of at most 18 digits (and possible leading minus sign).
+     *<p>
+     * NOTE: method DOES NOT verify that the contents actually are a valid
+     * Java {@code long} value (either regarding format, or value range): caller
+     * MUST validate that.
+     *
+     * @param neg Whether contents start with a minus sign
+     *
+     * @return Buffered text value parsed as an {@code long} using
+     *   {@link NumberInput#parseLong(String)} method (which does NOT validate input)
+     *
+     * @since 2.9
+     */
+    public long contentsAsLong(boolean neg) {
+        if ((_inputStart >= 0) && (_inputBuffer != null)) {
+            if (neg) {
+                return -NumberInput.parseLong(_inputBuffer, _inputStart+1, _inputLen-1);
+            }
+            return NumberInput.parseLong(_inputBuffer, _inputStart, _inputLen);
+        }
+        if (neg) {
+            return -NumberInput.parseLong(_currentSegment, 1, _currentSize-1);
+        }
+        return NumberInput.parseLong(_currentSegment, 0, _currentSize);
+    }
+
+    /**
+     * Accessor that will write buffered contents using given {@link Writer}.
+     *
+     * @param w Writer to use for writing out buffered content
+     *
+     * @return Number of characters written (same as {@link #size()})
+     *
+     * @throws IOException If write using {@link Writer} parameter fails
+     *
      * @since 2.8
      */
     public int contentsToWriter(Writer w) throws IOException
@@ -494,10 +721,8 @@ public final class TextBuffer
             start += max;
             len -= max;
         }
-        /* And then allocate new segment; we are guaranteed to now
-         * have enough room in segment.
-         */
-        // Except, as per [Issue-24], not for HUGE appends... so:
+        // And then allocate new segment; we are guaranteed to now
+        // have enough room in segment.
         do {
             expand(len);
             int amount = Math.min(_currentSegment.length, len);
@@ -531,10 +756,8 @@ public final class TextBuffer
             len -= max;
             offset += max;
         }
-        /* And then allocate new segment; we are guaranteed to now
-         * have enough room in segment.
-         */
-        // Except, as per [Issue-24], not for HUGE appends... so:
+        // And then allocate new segment; we are guaranteed to now
+        // have enough room in segment.
         do {
             expand(len);
             int amount = Math.min(_currentSegment.length, len);
@@ -597,6 +820,15 @@ public final class TextBuffer
     public void setCurrentLength(int len) { _currentSize = len; }
 
     /**
+     * Convenience method that finishes the current active content segment
+     * (by specifying how many characters within consists of valid content)
+     * and aggregates and returns resulting contents (similar to a call
+     * to {@link #contentsAsString()}).
+     *
+     * @param len Length of content (in characters) of the current active segment
+     *
+     * @return String that contains all buffered content
+     *
      * @since 2.6
      */
     public String setCurrentAndReturn(int len) {
@@ -638,6 +870,9 @@ public final class TextBuffer
      * Method called to expand size of the current segment, to
      * accommodate for more contiguous content. Usually only
      * used when parsing tokens like names if even then.
+     * Method will both expand the segment and return it
+     *
+     * @return Expanded current segment
      */
     public char[] expandCurrentSegment()
     {
@@ -659,7 +894,9 @@ public final class TextBuffer
      * 
      * @param minSize Required minimum strength of the current segment
      *
-     * @since 2.4.0
+     * @return Expanded current segment
+     *
+     * @since 2.4
      */
     public char[] expandCurrentSegment(int minSize) {
         char[] curr = _currentSegment;
@@ -712,10 +949,7 @@ public final class TextBuffer
         _currentSize = sharedLen;
     }
 
-    /**
-     * Method called when current segment is full, to allocate new
-     * segment.
-     */
+    // Method called when current segment is full, to allocate new segment.
     private void expand(int minNewSegmentSize)
     {
         // First, let's move current segment to segment list:
