@@ -10,6 +10,12 @@ import com.fasterxml.jackson.core.util.DefaultPrettyPrinter;
  */
 public class TestJDKSerializability extends BaseTest
 {
+    /*
+    /**********************************************************************
+    /* Main factory type(s)
+    /**********************************************************************
+     */
+
     public void testJsonFactorySerializable() throws Exception
     {
         JsonFactory f = new JsonFactory();
@@ -26,6 +32,12 @@ public class TestJDKSerializability extends BaseTest
         assertEquals(origJson, _copyJson(f2, origJson, true));
     }
 
+    /*
+    /**********************************************************************
+    /* Parser-related types
+    /**********************************************************************
+     */
+
     public void testBase64Variant() throws Exception
     {
         {
@@ -92,6 +104,12 @@ public class TestJDKSerializability extends BaseTest
         assertSame(ref2, ContentReference.unknown());
     }
 
+    /*
+    /**********************************************************************
+    /* Exception types
+    /**********************************************************************
+     */
+
     public void testParseException() throws Exception
     {
         JsonFactory jf = new JsonFactory();
@@ -127,7 +145,53 @@ public class TestJDKSerializability extends BaseTest
         JsonGenerationException result = jdkDeserialize(stuff);
         assertNotNull(result);
     }
-    
+
+    /*
+    /**********************************************************************
+    /* Misc other types
+    /**********************************************************************
+     */
+
+    public void testPointerSerializationNonEmpty() throws Exception
+    {
+        // First, see that we can write and read a general JsonPointer
+        final String INPUT = "/Image/15/name";
+        JsonPointer original = JsonPointer.compile(INPUT);
+        byte[] ser = jdkSerialize(original);
+        JsonPointer copy = jdkDeserialize(ser);
+        assertNotSame(copy, original);
+        assertEquals(original, copy);
+        assertEquals(original.hashCode(), copy.hashCode());
+
+        // 11-Oct-2022, tatu: Let's verify sub-path serializations too,
+        //   since 2.14 has rewritten internal implementation
+        JsonPointer branch = original.tail();
+        assertEquals("/15/name", branch.toString());
+        ser = jdkSerialize(branch);
+        copy = jdkDeserialize(ser);
+        assertEquals("/15/name", copy.toString());
+        assertEquals(branch, copy);
+        assertEquals(branch.hashCode(), copy.hashCode());
+
+        JsonPointer leaf = branch.tail();
+        assertEquals("/name", leaf.toString());
+        ser = jdkSerialize(leaf);
+        copy = jdkDeserialize(ser);
+        assertEquals("/name", copy.toString());
+        assertEquals(leaf, copy);
+        assertEquals(leaf.hashCode(), copy.hashCode());
+    }
+
+    public void testPointerSerializationEmpty() throws Exception
+    {
+        // and then verify that "empty" instance gets canonicalized
+        final JsonPointer emptyP = JsonPointer.empty();
+        byte[] ser = jdkSerialize(emptyP);
+        JsonPointer result = jdkDeserialize(ser);
+        assertSame("Should get same 'empty' instance when JDK serialize+deserialize",
+                emptyP, result);
+    }
+
     /*
     /**********************************************************
     /* Helper methods
