@@ -5,6 +5,7 @@ import java.io.IOException;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.core.testsupport.AsyncReaderWrapper;
 import com.fasterxml.jackson.core.testsupport.AsyncReaderWrapperForByteArray;
+import com.fasterxml.jackson.core.testsupport.AsyncReaderWrapperForByteBuffer;
 
 public abstract class AsyncTestBase extends BaseTest
 {
@@ -23,6 +24,14 @@ public abstract class AsyncTestBase extends BaseTest
                 bytesPerRead, bytes, padding);
     }
 
+    public static AsyncReaderWrapper asyncForByteBuffer(JsonFactory f,
+                                                        int bytesPerRead,
+                                                        byte[] bytes, int padding) throws IOException
+    {
+        return new AsyncReaderWrapperForByteBuffer(f.createNonBlockingByteBufferParser(),
+                bytesPerRead, bytes, padding);
+    }
+
     protected static String spaces(int count) 
     {
         return SPACES.substring(0, Math.min(SPACES.length(), count));
