@@ -85,7 +85,7 @@ public class Base64CodecTest
         byte[] decoded = std.decode(encoded);
         Assert.assertArrayEquals(input, decoded);
 
-        assertEquals(quote(encoded), std.encode(input, true));
+        assertEquals(q(encoded), std.encode(input, true));
 
         // [core#414]: check white-space allow too
         decoded = std.decode("\n"+encoded);
