@@ -141,14 +141,14 @@ public class GeneratorFeaturesTest
         JsonGenerator g = f.createGenerator(sw);
         g.writeNumber(ENG);
         g.close();
-        assertEquals(quote("100"), sw.toString());
+        assertEquals(q("100"), sw.toString());
 
         // also, as bytes
         ByteArrayOutputStream bos = new ByteArrayOutputStream();
         g = f.createGenerator(bos);
         g.writeNumber(ENG);
         g.close();
-        assertEquals(quote("100"), bos.toString("UTF-8"));
+        assertEquals(q("100"), utf8String(bos));
     }
 
     // [core#315]
@@ -234,7 +234,7 @@ public class GeneratorFeaturesTest
         g.writeEndArray();
         g.close();
 
-        return useBytes ? bytes.toString("UTF-8") : sw.toString();
+        return useBytes ? utf8String(bytes) : sw.toString();
     }
 
     // for [core#246]
@@ -283,7 +283,7 @@ public class GeneratorFeaturesTest
         gen.writeEndObject();
         gen.close();
 
-        String json = useBytes ? bytes.toString("UTF-8") : sw.toString();
+        String json = useBytes ? utf8String(bytes) : sw.toString();
         assertEquals(exp, json);
     }
 
@@ -354,4 +354,45 @@ public class GeneratorFeaturesTest
             assertEquals("{\"double\":NaN} {\"float\":NaN}", result);
         }
     }
+
+    // [core#717]: configurable hex digits; lower-case
+    public void testHexLowercase() throws Exception {
+        JsonFactory f = JsonFactory.builder()
+                .disable(JsonWriteFeature.WRITE_HEX_UPPER_CASE)
+                .build();
+        _testHexOutput(f, false, "\u001b", q("\\u001b"));
+        _testHexOutput(f, true, "\u001b", q("\\u001b"));
+    }
+
+    // [core#717]: configurable hex digits; upper-case (default)
+    public void testHexUppercase() throws Exception
+    {
+        JsonFactory f = JsonFactory.builder()
+                .enable(JsonWriteFeature.WRITE_HEX_UPPER_CASE)
+                .build();
+        _testHexOutput(f, false, "\u001b", q("\\u001B"));
+        _testHexOutput(f, true, "\u001b", q("\\u001B"));
+    }
+
+    private void _testHexOutput(JsonFactory f, boolean useBytes,
+            String input, String exp) throws Exception
+    {
+        ByteArrayOutputStream bytes = new ByteArrayOutputStream();
+        StringWriter sw = new StringWriter();
+        JsonGenerator g;
+        if (useBytes) {
+            g = f.createGenerator(bytes);
+        } else {
+            g = f.createGenerator(sw);
+        }
+
+        g.writeString(input);
+        g.flush();
+        g.close();
+
+        String result = useBytes ? utf8String(bytes) : sw.toString();
+        assertEquals(exp, result);
+
+        g.close();
+    }
 }
